//RS485 And CAN
#include <stdio.h>
#include <strings.h>
#include <unistd.h>
//RS485
#include <wiringSerial.h>
#include <wiringPi.h>
//CAN
#include <string.h>
#include <stdlib.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/can.h>
#include <linux/can/raw.h>


//RS485
#define EN_485 4
#define Baudrate 115200             //RS485 Baudrate
//CAN 
#define CAN_ID 0x123          //CAN ID
int Number_Received=0;

//Data to be sent by the RS485
unsigned char data[] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88};
int fd;

//CAN
int s;
struct can_frame frame;

int RS485_INIT(void)
{
    if(wiringPiSetupGpio() < 0) { //use BCM2835 Pin number table
        printf("set wiringPi lib failed	!!! \r\n");
        return 1;
    } else {
        printf("set wiringPi lib success  !!! \r\n");
    }
    pinMode(EN_485, OUTPUT);
    digitalWrite(EN_485,HIGH);

    if((fd = serialOpen ("/dev/ttyS0",Baudrate)) < 0) {
        printf("serial err\n");
        return -1;
    }

    serialFlush(fd);
    serialPrintf(fd,"\r");
    return 0;
}
//RS485 Send data
int RS485_Send(void)
{
    int i = 0;
    int dataSize = sizeof(data) / sizeof(data[0]); 
    printf("RS485 Send Data:\n");
    for (i = 0; i < dataSize; i++) {
        printf("RS485 Send Data[%d]:%X\n",i,data[i]);
        serialPutchar(fd,data[i]);
    }
    Number_Received++;
    return 0;
}
//CAN Receive data
int CAN_INIT(void)
{
    
    int ret;
    struct sockaddr_can addr;
    struct ifreq ifr;
    memset(&frame, 0, sizeof(struct can_frame));
    
    system("sudo ip link set can0 type can bitrate 250000");
    system("sudo ifconfig can0 up");
    printf("this is a can receive demo\r\n");
    
    //1.Create socket
    s = socket(PF_CAN, SOCK_RAW, CAN_RAW);
    if (s < 0) {
        perror("socket PF_CAN failed");
        return 1;
    }
    
    //2.Specify can0 device
    strcpy(ifr.ifr_name, "can0");
    ret = ioctl(s, SIOCGIFINDEX, &ifr);
    if (ret < 0) {
        perror("ioctl failed");
        return 1;
    }
    

    //3.Bind the socket to can0
    addr.can_family = PF_CAN;
    addr.can_ifindex = ifr.ifr_ifindex;
    ret = bind(s, (struct sockaddr *)&addr, sizeof(addr));
    if (ret < 0) {
        perror("bind failed");
        return 1;
    }
   

    //4.Define receive rules
    struct can_filter rfilter[1];
    rfilter[0].can_id = CAN_ID;
    rfilter[0].can_mask = CAN_SFF_MASK;
    setsockopt(s, SOL_CAN_RAW, CAN_RAW_FILTER, &rfilter, sizeof(rfilter));
    
    return 0;

}
int CAN_Read(void)
{
    int nbytes = 0,ret = 0;
    fd_set rdfs;
    FD_ZERO(&rdfs);         //Clear the file descriptor collection rdfs
    FD_SET(s, &rdfs);       //Add file descriptor s to file descriptor rdfs

    struct timeval timeout;
    timeout.tv_sec = 0;     // If none of the file descriptors are ready, the program will wait up to 1 second and then time out and return
    timeout.tv_usec = 0;    //The microsecond portion of the timeout,Set to 0 to ensure that the timeout precision is measured in seconds rather than microseconds. .
    
    ret = select(s + 1, &rdfs, NULL, NULL, &timeout);   //Detects whether data is received
    if (ret > 0 && FD_ISSET(s, &rdfs)) {                //The data is received and the file descriptor s is ready
        nbytes = read(s, &frame, sizeof(frame));
        if(nbytes > 0) {
            printf("\r\nCAN Read Data:\r\n");
            printf("can_id = 0x%X\r\ncan_dlc = %d \r\n", frame.can_id & 0x7FFFFFFF, frame.can_dlc);
            frame.can_id=0;
            int i = 0,Len=0;
            Len=sizeof(frame.data);
            for(i = 0; i < Len; i++)
                printf("CAN Read data[%d] = %x\r\n", i, frame.data[i]);
        }

    }
    else if(ret != 0)
        printf("Error in select()\r\n");
        
    return 0;
}
int main(void)
{
    int x = 0,y=0;
    // Close the socket and can0
    if(CAN_INIT() == 0 && RS485_INIT() == 0)
    {    
        while(1)
        {
            for(x = 0;x < 10001;x++)
            {
                if(x == 10000)
                {
                    y++;
                    if(y==1)
                    {
                        printf("======================== NEW Data %d =========================\r\n",Number_Received);
                        printf("Count Down :");
                        fflush(stdout);         // Refresh the buffer to ensure that data is sent in real time
                    }
                    if(y%20 == 0){
                        printf("%d ", 6 - y/20);
                        fflush(stdout);         // Refresh the buffer to ensure that data is sent in real time
                    }
                    if(y==100)
                    {
                        y=0;
                        printf("\r\n");
                        RS485_Send();
                    }  
                }
                CAN_Read(); 
            }
        }
    }
    else
        printf("INIT err");
    return 0;
}


